'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ifExists = undefined;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _util = require('util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const pstat = _util2.default.promisify(_fs2.default.stat);
const throwErr = err => {
  throw new Error(`qm-fs-ifExists - ifExists(): ${err.message}`);
};
const noENOENTexeption = err => err.code === 'ENOENT' ? false : throwErr(err); //TODOC: if ENOENT returns false

function ifExists(filePath, options = {}) {
  const {
    asBoolean = true,
    synchronous = false
  } = options;
  const result = synchronous ? existsSync(filePath, asBoolean) : _promise2.default.resolve().then(() => existsAsync(filePath, asBoolean)).catch(noENOENTexeption);
  return result;
}

async function existsAsync(filePath, asBoolean) {
  const rawStat = await pstat(filePath);
  const statMachined = fsStatObjectMachine(rawStat);
  const result = returnBoolOrVerbose(asBoolean, statMachined);
  return result;
}

function existsSync(filePath, asBoolean) {
  try {
    const rawStat = _fs2.default.statSync(filePath);
    const statMachined = fsStatObjectMachine(rawStat);
    const result = returnBoolOrVerbose(asBoolean, statMachined);
    return result;
  } catch (err) {
    return noENOENTexeption(err);
  }
}

function fsStatObjectMachine(statObj) {
  const notFound = statObj || 'notFound';
  const someResults = statObj && statObj.isDirectory() ? 'dir' : statObj.isFile() ? 'file' : 'other';
  return statObj ? someResults : notFound;
}

function returnBoolOrVerbose(asBoolean, statObj) {
  const result = asBoolean ? verboseToBoolean(statObj) : statObj;
  return result;
}

function verboseToBoolean(string) {
  return string === 'dir' || string === 'file' || string === 'other' ? true : false;
}

exports.ifExists = ifExists;