import fs from 'fs';
import util from 'util';

const pstat = util.promisify(fs.stat);
const throwErr = err => { throw new Error(`qm-fs-ifExists - ifExists(): ${err.message}`);};
const noENOENTexeption = err => err.code === 'ENOENT' ? false : throwErr(err); //TODOC: if ENOENT returns false

function ifExists(filePath, options = {}) {
  const {
    asBoolean = true,
    synchronous = false,
  } = options;
  const result = synchronous ?
    existsSync(filePath, asBoolean) :
    Promise
      .resolve()
      .then(() => existsAsync(filePath, asBoolean) )
      .catch(noENOENTexeption);
  return result;
}

async function existsAsync (filePath, asBoolean) {
  const rawStat = await pstat(filePath);
  const statMachined = fsStatObjectMachine(rawStat);
  const result = returnBoolOrVerbose(asBoolean, statMachined);
  return result;
}

function existsSync (filePath, asBoolean) {
  try {
    const rawStat = fs.statSync(filePath);
    const statMachined = fsStatObjectMachine(rawStat);
    const result = returnBoolOrVerbose(asBoolean, statMachined);
    return result;
  } catch (err) {
    return noENOENTexeption(err);
  }
}

function fsStatObjectMachine(statObj) {
  const notFound = statObj || 'notFound';
  const someResults = statObj && statObj.isDirectory() ? 'dir' :
    statObj.isFile() ? 'file' :
      'other';
  return statObj ? someResults : notFound;
}

function returnBoolOrVerbose (asBoolean, statObj) {
  const result = asBoolean ?
    verboseToBoolean(statObj) :
    statObj;
  return result;
}

function verboseToBoolean(string) {
  return string === 'dir' || string === 'file' || string === 'other' ? true :
    false;
}

export { ifExists };